@@ -133,9 +133,16 @@ Change an interface property::
      .interfaces['eth0']
      .set('state', 'up')
      .set('address', '00:11:22:33:44:55')
-     .commit()
+     .commit())
     # ---> <---  NDB waits here for the changes to be applied
 
+    # same as above, but using properties as argument names
+    (ndb
+     .interfaces['eth0']
+     .set(state='up')
+     .set(address='00:11:22:33:44:55')
+     .commit())
+
     # ... or with another syntax
     with ndb.interfaces['eth0'] as i:
         i['state'] = 'up'
@@ -171,9 +178,11 @@ from pyroute2.ndb.messages import (cmsg,
                                    cmsg_event,
                                    cmsg_failed,
                                    cmsg_sstart)
-from pyroute2.ndb.source import Source
+from pyroute2.ndb.source import (Source,
+                                 SourceProxy)
 from pyroute2.ndb.auth_manager import check_auth
 from pyroute2.ndb.auth_manager import AuthManager
+from pyroute2.ndb.objects import RSLV_DELETE
 from pyroute2.ndb.objects.interface import Interface
 from pyroute2.ndb.objects.interface import Vlan
 from pyroute2.ndb.objects.address import Address
@@ -184,6 +193,7 @@ from pyroute2.ndb.objects.netns import NetNS
 # from pyroute2.ndb.query import Query
 from pyroute2.ndb.report import (RecordSet,
                                  Record)
+from pyroute2.netlink import nlmsg_base
 try:
     from urlparse import urlparse
 except ImportError:
@@ -200,6 +210,9 @@ except ImportError:
     psycopg2 = None
 
 log = logging.getLogger(__name__)
+_sql_adapters_lock = threading.Lock()
+_sql_adapters_psycopg2_registered = False
+_sql_adapters_sqlite3_registered = False
 
 
 def target_adapter(value):
@@ -218,11 +231,83 @@ class PostgreSQLAdapter(object):
         return "'%s'" % json.dumps(self.obj)
 
 
-sqlite3.register_adapter(list, target_adapter)
-sqlite3.register_adapter(dict, target_adapter)
-if psycopg2 is not None:
-    psycopg2.extensions.register_adapter(list, PostgreSQLAdapter)
-    psycopg2.extensions.register_adapter(dict, PostgreSQLAdapter)
+def register_sqlite3_adapters():
+    global _sql_adapters_lock
+    global _sql_adapters_sqlite3_registered
+    with _sql_adapters_lock:
+        if not _sql_adapters_sqlite3_registered:
+            _sql_adapters_sqlite3_registered = True
+            sqlite3.register_adapter(list, target_adapter)
+            sqlite3.register_adapter(dict, target_adapter)
+
+
+def regsiter_postgres_adapters():
+    global _sql_adapters_lock
+    global _sql_adapters_psycopg2_registered
+    with _sql_adapters_lock:
+        if psycopg2 is not None and not _sql_adapters_psycopg2_registered:
+            _sql_adapters_psycopg2_registered = True
+            psycopg2.extensions.register_adapter(list, PostgreSQLAdapter)
+            psycopg2.extensions.register_adapter(dict, PostgreSQLAdapter)
+
+
+class Transaction(object):
+
+    def __init__(self, log):
+        self.queue = []
+        self.event = threading.Event()
+        self.event.clear()
+        self.log = log.channel('transaction.%s' % id(self))
+        self.log.debug('begin transaction')
+
+    def push(self, *argv):
+        for obj in argv:
+            self.log.debug('queue %s' % type(obj))
+            self.queue.append(obj)
+        return self
+
+    def append(self, obj):
+        self.log.debug('queue %s' % type(obj))
+        self.push(obj)
+        return self
+
+    def pop(self, index=-1):
+        self.log.debug('pop %s' % index)
+        self.queue.pop(index)
+        return self
+
+    def insert(self, index, obj):
+        self.log.debug('insert %i %s' % (index, type(obj)))
+        self.queue.insert(index, obj)
+        return self
+
+    def cancel(self):
+        self.log.debug('cancel transaction')
+        self.queue = []
+        return self
+
+    def wait(self):
+        return self.event.wait()
+
+    def done(self):
+        return self.event.is_set()
+
+    def commit(self):
+        self.log.debug('commit')
+        rollbacks = []
+        for obj in self.queue:
+            rollbacks.append(obj)
+            try:
+                obj.commit()
+            except Exception:
+                for rb in reversed(rollbacks):
+                    try:
+                        rb.rollback()
+                    except Exception as e:
+                        self.log.warning('ignore rollback exception: %s' % e)
+                raise
+        self.event.set()
+        return self
 
 
 class View(dict):
@@ -288,15 +373,38 @@ class View(dict):
         except KeyError:
             return None
 
-    @cli.change_pointer
-    def create(self, *argspec, **kwspec):
+    def template(self, key, table=None):
         if self.chain:
             context = self.chain.context
         else:
             context = {}
-        spec = (self
-                .classes[self.table]
-                .adjust_spec(kwspec or argspec[0], context))
+        iclass = self.classes[table or self.table]
+
+        spec = (iclass
+                .new_spec(key, self.default_target)
+                .load_context(context)
+                .get_spec)
+
+        return iclass(self,
+                      spec,
+                      load=False,
+                      master=self.chain,
+                      auth_managers=self.auth_managers)
+
+    @cli.change_pointer
+    def create(self, *argspec, **kwspec):
+        iclass = self.classes[self.table]
+
+        if self.chain:
+            context = self.chain.context
+        else:
+            context = {}
+
+        spec = (iclass
+                .new_spec(kwspec or argspec[0], self.default_target)
+                .load_context(context)
+                .get_spec)
+
         if self.chain:
             spec['ndb_chain'] = self.chain
         spec['create'] = True
@@ -317,6 +425,8 @@ class View(dict):
 
     def wait(self, **spec):
         ret = None
+        timeout = spec.pop('timeout', None)
+        ctime = time.time()
 
         # install a limited events queue -- for a possible immediate reaction
         evq = queue.Queue(maxsize=100)
@@ -352,6 +462,9 @@ class View(dict):
             ret = None
 
         while ret is None:
+            if timeout is not None:
+                if ctime + timeout < time.time():
+                    break
             try:
                 target, msg = evq.get(timeout=1)
             except queue.Empty:
@@ -390,24 +503,14 @@ class View(dict):
         del evq
         del hdl
         gc.collect()
+        if ret is None:
+            raise TimeoutError()
         return ret
 
     @check_auth('obj:read')
     def __getitem__(self, key, table=None):
 
-        if self.chain:
-            context = self.chain.context
-        else:
-            context = {}
-        iclass = self.classes[table or self.table]
-        if isinstance(key, Record):
-            key = key._as_dict()
-        key = iclass.adjust_spec(key, context)
-        ret = iclass(self,
-                     key,
-                     load=False,
-                     master=self.chain,
-                     auth_managers=self.auth_managers)
+        ret = self.template(key, table)
 
         # rtnl_object.key() returns a dcitionary that can not
         # be used as a cache key. Create here a tuple from it.
@@ -445,13 +548,55 @@ class View(dict):
             return ret
         else:
             # Cache only existing objects
-            if ret.load_sql():
+            if self.exists(key):
+                ret.load_sql()
                 self.log.debug('cache add %s' % (cache_key, ))
                 self.cache[cache_key] = ret
 
         ret.register()
         return ret
 
+    def exists(self, key, table=None):
+        '''
+        Check if the specified object exists in the database::
+
+            ndb.interfaces.exists('eth0')
+            ndb.interfaces.exists({'ifname': 'eth0', 'target': 'localhost'})
+            ndb.addresses.exists('127.0.0.1/8')
+        '''
+        iclass = self.classes[self.table]
+        key = iclass.new_spec(key, self.default_target).get_spec
+
+        iclass.resolve(view=self,
+                       spec=key,
+                       fields=iclass.resolve_fields,
+                       policy=RSLV_DELETE)
+
+        table = table or self.table
+        schema = self.ndb.schema
+        names = schema.compiled[self.table]['all_names']
+
+        self.log.debug('check if the key %s exists in table %s' %
+                       (key, table))
+        keys = []
+        values = []
+        for name, value in key.items():
+            nla_name = iclass.name2nla(name)
+            if nla_name in names:
+                name = nla_name
+            if value is not None and name in names:
+                keys.append('f_%s = %s' % (name, schema.plch))
+                values.append(value)
+        spec = (schema
+                .fetchone('SELECT * FROM %s WHERE %s' %
+                          (self.table, ' AND '.join(keys)), values))
+        if spec is not None:
+            self.log.debug('exists')
+            return True
+        else:
+            self.log.debug('not exists')
+            return False
+
     def __setitem__(self, key, value):
         raise NotImplementedError()
 
@@ -461,6 +606,9 @@ class View(dict):
     def __iter__(self):
         return self.keys()
 
+    def __contains__(self, key):
+        return key in self.dump()
+
     @check_auth('obj:list')
     def keys(self):
         for record in self.dump():
@@ -493,7 +641,7 @@ class View(dict):
     def _native(self, dump):
         fnames = next(dump)
         for record in dump:
-            yield Record(fnames, record)
+            yield Record(fnames, record, self.classes[self.table])
 
     @cli.show_result
     @check_auth('obj:list')
@@ -510,11 +658,15 @@ class View(dict):
 
 class SourcesView(View):
 
-    def __init__(self, ndb):
+    def __init__(self, ndb, auth_managers=None):
         super(SourcesView, self).__init__(ndb, 'sources')
         self.classes['sources'] = Source
         self.cache = {}
+        self.proxy = {}
         self.lock = threading.Lock()
+        if auth_managers is None:
+            auth_managers = []
+        self.auth_managers = auth_managers
 
     def async_add(self, **spec):
         spec = dict(Source.defaults(spec))
@@ -540,6 +692,7 @@ class SourcesView(View):
                 source.close(code=code, sync=sync)
                 return self.cache.pop(target)
 
+    @check_auth('obj:list')
     def keys(self):
         for key in self.cache:
             yield key
@@ -561,7 +714,14 @@ class SourcesView(View):
         else:
             raise ValueError('key format not supported')
 
-        return self.cache[target]
+        if target in self.cache:
+            return self.cache[target]
+        elif target in self.proxy:
+            return self.proxy[target]
+        else:
+            proxy = SourceProxy(self.ndb, target)
+            self.proxy[target] = proxy
+            return proxy
 
 
 class Log(object):
@@ -694,13 +854,13 @@ class AuthProxy(object):
         self._ndb = ndb
         self._auth_managers = auth_managers
 
-        for spec in (('interfaces', 'localhost'),
-                     ('addresses', 'localhost'),
-                     ('routes', 'localhost'),
-                     ('neighbours', 'localhost'),
-                     ('rules', 'localhost'),
-                     ('netns', 'nsmanager'),
-                     ('vlans', 'localhost')):
+        for spec in (('interfaces', ndb.localhost),
+                     ('addresses', ndb.localhost),
+                     ('routes', ndb.localhost),
+                     ('neighbours', ndb.localhost),
+                     ('rules', ndb.localhost),
+                     ('netns', ndb.nsmanager),
+                     ('vlans', ndb.localhost)):
             view = View(self._ndb,
                         spec[0],
                         default_target=spec[1],
@@ -710,15 +870,31 @@ class AuthProxy(object):
 
 class NDB(object):
 
+    @property
+    def nsmanager(self):
+        return '%s/nsmanager' % self.localhost
+
     def __init__(self,
                  sources=None,
+                 localhost='localhost',
                  db_provider='sqlite3',
                  db_spec=':memory:',
+                 db_cleanup=True,
                  rtnl_debug=False,
                  log=False,
                  auto_netns=False,
-                 libc=None):
+                 libc=None,
+                 messenger=None):
 
+        if db_provider == 'postgres':
+            db_provider = 'psycopg2'
+
+        if db_provider == 'sqlite3':
+            register_sqlite3_adapters()
+        elif db_provider == 'psycopg2':
+            regsiter_postgres_adapters()
+
+        self.localhost = localhost
         self.ctime = self.gctime = time.time()
         self.schema = None
         self.config = {}
@@ -731,9 +907,16 @@ class NDB(object):
         self._dbm_thread = None
         self._dbm_ready = threading.Event()
         self._dbm_shutdown = threading.Event()
+        self._db_cleanup = db_cleanup
         self._global_lock = threading.Lock()
         self._event_map = None
         self._event_queue = EventQueue(maxsize=100)
+        self.messenger = messenger
+        if messenger is not None:
+            self._mm_thread = threading.Thread(target=self.__mm__,
+                                               name='Messenger')
+            self._mm_thread.setDaemon(True)
+            self._mm_thread.start()
         #
         if log:
             if isinstance(log, basestring):
@@ -747,42 +930,51 @@ class NDB(object):
         #
         # fix sources prime
         if sources is None:
-            sources = [{'target': 'localhost',
+            sources = [{'target': self.localhost,
                         'kind': 'local',
                         'nlm_generator': 1}]
             if sys.platform.startswith('linux'):
-                sources.append({'target': 'nsmanager',
+                sources.append({'target': self.nsmanager,
                                 'kind': 'nsmanager'})
         elif not isinstance(sources, (list, tuple)):
             raise ValueError('sources format not supported')
 
-        self.sources = SourcesView(self)
+        for spec in sources:
+            if 'target' not in spec:
+                spec['target'] = self.localhost
+                break
+
+        am = AuthManager({'obj:list': True,
+                          'obj:read': True,
+                          'obj:modify': True},
+                         self.log.channel('auth'))
+        self.sources = SourcesView(self, auth_managers=[am])
+        self._call_registry = {}
         self._nl = sources
         self._db_provider = db_provider
         self._db_spec = db_spec
         self._db_rtnl_log = rtnl_debug
         atexit.register(self.close)
         self._dbm_ready.clear()
+        self._dbm_error = None
         self._dbm_autoload = set()
         self._dbm_thread = threading.Thread(target=self.__dbm__,
                                             name='NDB main loop')
         self._dbm_thread.setDaemon(True)
         self._dbm_thread.start()
         self._dbm_ready.wait()
+        if self._dbm_error is not None:
+            raise self._dbm_error
         for event in tuple(self._dbm_autoload):
             event.wait()
         self._dbm_autoload = None
-        am = AuthManager({'obj:list': True,
-                          'obj:read': True,
-                          'obj:modify': True},
-                         self.log.channel('auth'))
-        for spec in (('interfaces', 'localhost'),
-                     ('addresses', 'localhost'),
-                     ('routes', 'localhost'),
-                     ('neighbours', 'localhost'),
-                     ('rules', 'localhost'),
-                     ('netns', 'nsmanager'),
-                     ('vlans', 'localhost')):
+        for spec in (('interfaces', self.localhost),
+                     ('addresses', self.localhost),
+                     ('routes', self.localhost),
+                     ('neighbours', self.localhost),
+                     ('rules', self.localhost),
+                     ('netns', self.nsmanager),
+                     ('vlans', self.localhost)):
             view = View(self,
                         spec[0],
                         default_target=spec[1],
@@ -799,6 +991,9 @@ class NDB(object):
     def __exit__(self, exc_type, exc_value, traceback):
         self.close()
 
+    def begin(self):
+        return Transaction(self.log)
+
     def auth_proxy(self, auth_manager):
         return AuthProxy(self, [auth_manager, ])
 
@@ -831,6 +1026,47 @@ class NDB(object):
             self._event_queue.bypass((cmsg(None, ShutdownException()), ))
             self._dbm_thread.join()
 
+    def reload(self, kinds=None):
+        for source in self.sources.values():
+            if kinds is not None and source.kind in kinds:
+                source.restart()
+
+    def __mm__(self):
+        # notify neighbours by sending hello
+        for peer in self.messenger.transport.peers:
+            peer.hello()
+        # receive events
+        for msg in self.messenger:
+            if msg['type'] == 'system' and msg['data'] == 'HELLO':
+                for peer in self.messenger.transport.peers:
+                    peer.last_exception_time = 0
+                self.reload(kinds=['local', 'netns', 'remote'])
+            elif msg['type'] == 'transport':
+                message = msg['data'][0](data=msg['data'][1])
+                message.decode()
+                message['header']['target'] = msg['target']
+                self._event_queue.put((message, ))
+            elif msg['type'] == 'response':
+                if msg['call_id'] in self._call_registry:
+                    event = self._call_registry.pop(msg['call_id'])
+                    self._call_registry[msg['call_id']] = msg
+                    event.set()
+            elif msg['type'] == 'api':
+                if msg['target'] in self.messenger.targets:
+                    try:
+                        ret = self.sources[msg['target']].api(msg['name'],
+                                                              *msg['argv'],
+                                                              **msg['kwarg'])
+                        self.messenger.emit({'type': 'response',
+                                             'call_id': msg['call_id'],
+                                             'return': ret})
+                    except Exception as e:
+                        self.messenger.emit({'type': 'response',
+                                             'call_id': msg['call_id'],
+                                             'exception': e})
+            else:
+                self.log.warning('unknown protocol via messenger')
+
     def __dbm__(self):
 
         def default_handler(target, event):
@@ -856,16 +1092,22 @@ class NDB(object):
 
         event_queue = self._event_queue
 
-        if self._db_provider == 'sqlite3':
-            self._db = sqlite3.connect(self._db_spec)
-        elif self._db_provider == 'psycopg2':
-            self._db = psycopg2.connect(**self._db_spec)
+        try:
+            with _sql_adapters_lock:
+                if self._db_provider == 'sqlite3':
+                    self._db = sqlite3.connect(self._db_spec)
+                elif self._db_provider == 'psycopg2':
+                    self._db = psycopg2.connect(**self._db_spec)
 
-        self.schema = schema.init(self,
-                                  self._db,
-                                  self._db_provider,
-                                  self._db_rtnl_log,
-                                  id(threading.current_thread()))
+            self.schema = schema.init(self,
+                                      self._db,
+                                      self._db_provider,
+                                      self._db_rtnl_log,
+                                      id(threading.current_thread()))
+        except Exception as e:
+            self._dbm_error = e
+            self._dbm_ready.set()
+            return
 
         for spec in self._nl:
             spec['event'] = None
@@ -884,6 +1126,24 @@ class NDB(object):
                 for event in events:
                     handlers = event_map.get(event.__class__,
                                              [default_handler, ])
+                    if self.messenger is not None and\
+                            (event
+                             .get('header', {})
+                             .get('target', None) in self.messenger.targets):
+                        if isinstance(event, nlmsg_base):
+                            if event.data is not None:
+                                data = event.data[event.offset:
+                                                  event.offset + event.length]
+                            else:
+                                event.reset()
+                                event.encode()
+                                data = event.data
+                            data = (type(event), data)
+                            tgt = event['header']['target']
+                            self.messenger.emit({'type': 'transport',
+                                                 'target': tgt,
+                                                 'data': data})
+
                     for handler in tuple(handlers):
                         try:
                             target = event['header']['target']
@@ -928,9 +1188,16 @@ class NDB(object):
             if source is not None and source.th is not None:
                 source.shutdown.set()
                 source.th.join()
-                self.log.debug('flush DB for the target %s' % target)
-                self.schema.flush(target)
+                if self._db_cleanup:
+                    self.log.debug('X flush DB for the target %s' % target)
+                    self.schema.flush(target)
+                else:
+                    self.log.debug('leave DB for debug')
 
         # close the database
         self.schema.commit()
         self.schema.close()
+
+        # close the logging
+        for handler in self.log.logger.handlers:
+            handler.close()
