@@ -13,12 +13,15 @@ from pyroute2.common import basestring
 from pyroute2.netlink import nla
 from pyroute2.netlink import nlmsg
 from pyroute2.netlink import nlmsg_atoms
+from pyroute2.netlink import NLA_F_NESTED
 from pyroute2.netlink.rtnl.iw_event import iw_event
 from pyroute2.netlink.rtnl.ifinfmsg.plugins import (bond,
+                                                    geneve,
                                                     gtp,
                                                     ipvlan,
                                                     team,
                                                     tuntap,
+                                                    tun,
                                                     vlan,
                                                     vrf,
                                                     vti,
@@ -213,10 +216,12 @@ def load_plugins_by_pkg(pkg):
 data_plugins = {}
 
 for module in (bond,
+               geneve,
                gtp,
                ipvlan,
                team,
                tuntap,
+               tun,
                vlan,
                vrf,
                vti,
@@ -294,6 +299,7 @@ class protinfo_bridge(nla):
 
 
 class macvx_data(nla):
+    prefix = 'IFLA_'
     nla_map = (('IFLA_MACVLAN_UNSPEC', 'none'),
                ('IFLA_MACVLAN_MODE', 'mode'),
                ('IFLA_MACVLAN_FLAGS', 'flags'),
@@ -325,6 +331,7 @@ class macvx_data(nla):
 
 
 class iptnl_data(nla):
+    prefix = 'IFLA_'
     nla_map = (('IFLA_IPIP_UNSPEC', 'none'),
                ('IFLA_IPIP_LINK', 'uint32'),
                ('IFLA_IPIP_LOCAL', 'ip4addr'),
@@ -440,10 +447,17 @@ class ifinfbase(object):
                ('IFLA_XDP', 'hex'),
                ('IFLA_EVENT', 'hex'),
                ('IFLA_NEW_NETNSID', 'hex'),
-               ('IFLA_IF_NETNSID', 'hex'),
+               ('IFLA_IF_NETNSID', 'uint32'),
                ('IFLA_CARRIER_UP_COUNT', 'uint32'),
                ('IFLA_CARRIER_DOWN_COUNT', 'uint32'),
-               ('IFLA_NEW_IFINDEX', 'hex'))
+               ('IFLA_NEW_IFINDEX', 'uint32'),
+               ('IFLA_MIN_MTU', 'uint32'),
+               ('IFLA_MAX_MTU', 'uint32'),
+               ('IFLA_PROP_LIST', 'proplist'),
+               ('IFLA_ALT_IFNAME', 'asciiz'),
+               ('IFLA_PERM_ADDRESS', 'hex'),
+               ('IFLA_PROTO_DOWN_REASON', 'hex'),
+               )
 
     @staticmethod
     def flags2names(flags, mask=0xffffffff):
@@ -490,6 +504,8 @@ class ifinfbase(object):
             if isinstance(self.value, int):
                 self['value'] = self.value
             else:
+                if isinstance(self.value, bytes):
+                    self.value = self.value.decode('utf-8')
                 if '/' in self.value:
                     netns_path = self.value
                 else:
@@ -503,6 +519,18 @@ class ifinfbase(object):
             if self.netns_fd is not None:
                 os.close(self.netns_fd)
 
+    class proplist(nla):
+        nla_flags = NLA_F_NESTED
+        # Proplist has currently only IFLA_ALT_IFNAME, but start at same
+        # index than IFLA_ALT_IFNAME in ifinfbase()
+        nla_map = (
+            (53, 'IFLA_ALT_IFNAME', 'asciiz'),
+        )
+
+        def altnames(self):
+            return (attr[1] for attr in self["attrs"]
+                    if attr[0] == "IFLA_ALT_IFNAME")
+
     class vflist(nla):
         nla_map = (('IFLA_VF_INFO_UNSPEC', 'none'),
                    ('IFLA_VF_INFO', 'vfinfo'))
@@ -761,6 +789,7 @@ class ifinfbase(object):
 
             class gre_flags(nla):
                 fields = [('value', '>H')]
+                sql_type = 'INTEGER'
 
                 def encode(self):
                     #
@@ -783,6 +812,7 @@ class ifinfbase(object):
             # Linux uses the same enum names for v6 and v4 (in if_tunnel.h);
             # Here we name them IFLA_IP6GRE_xxx instead to avoid conflicts
             # with gre_data above.
+            prefix = 'IFLA_'
             nla_map = (('IFLA_IP6GRE_UNSPEC', 'none'),
                        ('IFLA_IP6GRE_LINK', 'uint32'),
                        ('IFLA_IP6GRE_IFLAGS', 'uint16'),
